package cp;


import cp.db.DB;

import java.io.FileWriter;
import java.io.IOException;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;

public class Game {
  private Player white;
  private Player black;
  private Result result;
  private Games games;
  private int roundNr;
  private int boardNr;
  private int whiteLotNr, blackLotNr;

  public Game(Games games) throws ChessPairerException {
    this.games = games;
    result = new Result(this, Result.ResultType.NO_RESULT);
  }
  
  public Game(Games games, Player white, Player black) throws ChessPairerException {
    this.games = games;
    this.white = white;
    this.black = black;
    if (white != null) {
      this.whiteLotNr = white.getLotNr();
    }
    else {
      this.whiteLotNr = Integer.MAX_VALUE;
    }
    if (black != null) {
      this.blackLotNr = black.getLotNr();
    }
    else {
      this.blackLotNr = Integer.MAX_VALUE;
    }
    setResult();
  }

  public void setWhitePlayer(Player white) throws ChessPairerException {
    this.white = white;
    if (white != null) {
      this.whiteLotNr = white.getLotNr();
    }
    else {
      this.whiteLotNr = Integer.MAX_VALUE;
    }
    setResult();
  }

  public void setBlackPlayer(Player black) throws ChessPairerException {
    this.black = black;
    if (black != null) {
      this.blackLotNr = black.getLotNr();
    }
    else {
      this.blackLotNr = Integer.MAX_VALUE;
    }
    setResult();
  }
  
  private void setResult() throws ChessPairerException {
    if (white == null && black != null) {
      result = new Result(this, Result.ResultType.BLACK_WIN);
    }
    else if (black == null && white != null) {
      result = new Result(this, Result.ResultType.WHITE_WIN);
    }
    else {
      result = new Result(this, Result.ResultType.NO_RESULT);
    }
  }

  public boolean hasMet(Player a, Player b) {
    return (a == white && b == black || a == black && b == white);
  }
  
  public boolean plays(Player a) {
    return (a == white || a == black);
  }
  
  public Player getHighestPlayer() {
    if (white == null) {
      return black;
    }
    else if (black == null) {
      return white;
    }
    else if (whiteLotNr < blackLotNr) {
      return white;
    }
    else {
      return black;
    }
  }
  
  public Player getBlackPlayer() {
    return black;
  }
  
  public Player getWhitePlayer() {
    return white;
  }
  
  public Player getOpponent(Player player) {
    if (player == white) {
      return black;
    }
    else if (player == black) {
      return white;
    }
    return null;
  }
  
  void addToPlayers() {
    if (white != null) {
      white.add(this);
    }
    if (black != null) {
      black.add(this);
    }
  }
  
  void removeFromPlayers() {
    if (white != null) {
      white.remove(this);
    }
    if (black != null) {
      black.remove(this);
    }
  }
  
  void setResult(Result result) {
    this.result = result;
  }
  
  public Result getResult() {
    return result;
  }
  
  public double getPairingScore(Player player, int roundNr) {
    if (this.roundNr <= roundNr) {
      Tournament tournament = games.getTournament();
      boolean compensate = tournament.isCompensateWeakPlayerPP() && (result.getResultType() == Result.ResultType.POSTPONED);
      if (player == white) {
        Player black = getBlackPlayer();
        if (compensate && black != null && tournament.getRating(black)-tournament.getRating(player)>200) {
          return result.getScore(Result.ResultType.DRAW);
        }
        else {
          return result.getWhitePairingScore();
        }
      }
      else if (player == black) {
        Player white = getWhitePlayer();
        if (compensate && white != null && tournament.getRating(white)-tournament.getRating(player)>200) {
          return result.getScore(Result.ResultType.DRAW);
        }
        else {
          return result.getBlackPairingScore();
        }
      }
    }
    return 0;
  }
  
  public double getScore(Player player, int roundNr) {
    if (this.roundNr <= roundNr) {
      if (player == white) {
        return result.getWhiteScore();
      }
      else if (player == black) {
        return result.getBlackScore();
      }
    }
    return 0;
  }

  boolean hasResult() {
    return result.getResultType() != Result.ResultType.NO_RESULT;
  }


  void save(int roundNr, int boardNr) throws SQLException {
    Connection conn = DB.getConnection();
    String sql = "INSERT INTO tournamentgames " + 
                 "(tournament, round, boardnr, whiteplayer, blackplayer, resulttype, whitescore, blackscore, whiteplayerlotnr, blackplayerlotnr) " + 
                 "VALUES (?,?,?,?,?,?,?,?,?,?)";
    PreparedStatement s = conn.prepareStatement(sql);
    s.setInt(1, games.getTournament().getIndex());
    s.setInt(2, roundNr);
    s.setInt(3, boardNr);
    if (white != null) {
      s.setInt(4, white.getIndex());
      s.setInt(9, white.getLotNr());
    }
    else {
      s.setNull(4, Types.INTEGER);
      s.setNull(9, Types.INTEGER);
    }
    if (black != null) {
      s.setInt(5, black.getIndex());
      s.setInt(10, black.getLotNr());
    }
    else {
      s.setNull(5, Types.INTEGER);
      s.setNull(10, Types.INTEGER);
    }
    s.setInt(6, result.getResultType().ordinal());
    s.setDouble(7, result.getWhitePairingScore());
    s.setDouble(8, result.getBlackPairingScore());

    int rows = s.executeUpdate();
    if (rows != 1) {
      throw new SQLException(rows + " rader ndrades, frvntade 1 rad");
    }
    this.roundNr = roundNr;
    this.boardNr = boardNr;
  }

  void load(ResultSet rs, Players players) throws SQLException {
    white = players.findPlayer(rs.getInt("whiteplayer"));
    if (white != null) {
      white.add(this);
      whiteLotNr = rs.getInt("whiteplayerlotnr");
    }
    black = players.findPlayer(rs.getInt("blackplayer"));
    if (black != null) {
      black.add(this);
      blackLotNr = rs.getInt("blackplayerlotnr");
    }
    result = new Result(this, rs.getDouble("whitescore"), rs.getDouble("blackscore"), Result.ResultType.values()[rs.getInt("resulttype")]);    
    roundNr = rs.getInt("round");
    boardNr = rs.getInt("boardnr");
  }

  public void update() throws SQLException {
    Connection conn = DB.getConnection();
    String sql = "UPDATE tournamentgames " + 
                 "SET whiteplayer=?, blackplayer=?, resulttype=?, whitescore=?, blackscore=?, whiteplayerlotnr=?, blackplayerlotnr=? " + 
                 "WHERE tournament=? AND round=? AND boardnr=?";
    PreparedStatement s = conn.prepareStatement(sql);
    if (white != null) {
      s.setInt(1, white.getIndex());
      s.setInt(6, white.getLotNr());
    }
    else {
      s.setNull(1, Types.INTEGER);
      s.setNull(6, Types.INTEGER);
    }
    if (black != null) {
      s.setInt(2, black.getIndex());
      s.setInt(7, black.getLotNr());
    }
    else {
      s.setNull(2, Types.INTEGER);
      s.setNull(7, Types.INTEGER);
    }
    s.setInt(3, result.getResultType().ordinal());
    s.setDouble(4, result.getWhitePairingScore());
    s.setDouble(5, result.getBlackPairingScore());
    s.setInt(8, games.getTournament().getIndex());
    s.setInt(9, roundNr);
    s.setInt(10, boardNr);
    try {
      int rows = s.executeUpdate();
      if (rows != 1) {
        throw new SQLException(rows + " rader ndrades, frvntade 1 rad");
      }
      conn.commit();
    }
    catch (SQLException e) {
      conn.rollback();
      throw e;
    }
    s.close();
  }

  public Games getGames() {
    return games;
  }

  void setGames(Games games) {
    this.games = games;
  }

  public int getBoardNr() {
    return boardNr;
  }

  public int getRoundNr() {
    return roundNr;
  }

	void exportToLiveChess(FileWriter fileWriter, int roundNr, int board) throws IOException {
		if (white != null && black != null && result.getResultType() == Result.ResultType.NO_RESULT) {
		  Tournament tournament = this.getGames().getTournament();
		  String str = String.format("[Event \"%s\"][Round \"%d\"][White \"%s, %s\"][Black \"%s, %s\"][Board \"%d\"][WhiteElo \"%d\"][BlackElo \"%d\"]*\r\n", games.getTournament().getName(), roundNr, white.getLastName(), white.getFirstName(), black.getLastName(), black.getFirstName(), board, tournament.getRating(white), tournament.getRating(black));
			fileWriter.write(str);
		}
	}

  public boolean wonOnWO(Player player) {
    if (player == white) {
      return result.getResultType() == Result.ResultType.WHITE_WIN_WO;
    }
    else if (player == black) {
      return result.getResultType() == Result.ResultType.BLACK_WIN_WO;
    }
    return false;
  }
  
  /*
   * Is used by SSF to set how pairing score is calculated.
   */
  public void setRoundNr(int roundNr) {
       this.roundNr = roundNr;
  }

  int getWhiteLotNr() {
    return whiteLotNr;
  }

  int getBlackLotNr() {
    return blackLotNr;
  }
}
