package cp.gui.web;

import cp.ChessPairerException;
import cp.Club;
import cp.Player;
import cp.Players;
import cp.Tournament;
import cp.db.Settings;
import cp.gui.SelectPlayerDialog;
import cp.gui.SelectPlayerDialog.SSFUpdateResult;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.net.URL;
import java.nio.charset.Charset;
import java.sql.Date;
import java.sql.SQLException;
import javax.swing.JDialog;
import javax.swing.JOptionPane;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

public class SSF {
  
  /*
   * Updates tournament players via SSF registry.
   * If they are available player in the player pool, set tournament to null.
   */
  public static void updatePlayers(Players players, Tournament tournament, Settings settings, JDialog parent) throws SQLException, IOException, ChessPairerException {
    for (int i=0; i<players.size(); i++) {
      Player player = players.get(i);
      updatePlayer(player, tournament, settings, parent, true);
    }
  }

  /*
   * Updates players via SSF registry.
   * If it's an available player in the player pool, set tournament to null.
   */
  public static void updatePlayer(Player player, Tournament tournament, Settings settings, JDialog parent, boolean update) throws SQLException, IOException, ChessPairerException {
    int memberid = player.getSSFID();
    if (memberid == 0) {
      // Search for player
      SelectPlayerDialog selectPlayerDlg = new SelectPlayerDialog(parent);
      try {
        String url = String.format("http://member.schack.se/JSONMemberSearch?firstname=%s*&lastname=%s*", player.getFirstName(), player.getLastName());
        JSONObject obj = SSF.readJsonFromUrl(url);
        JSONArray array = obj.getJSONArray("members");
        int index = 0;
        if (array.length() > 1) {
          index = selectPlayerDlg.getPlayerIndex(player, array);
          SSFUpdateResult result = selectPlayerDlg.getResult();
          if (result == SSFUpdateResult.ABORTED) {
            throw new ChessPairerException("Aborted");
          }
          else if (result == SSFUpdateResult.MISSING) {
            return;
          }
        }
        if (array.length() > 0) {
          JSONObject o2 = array.getJSONObject(index);
          // Now search for tha member given ID
          memberid = o2.getInt("memberid");
        }
        else if (array.length() == 0) {
          JOptionPane.showMessageDialog(parent, "Kan inte hitta spelare " + player.getFirstName() + " " + player.getLastName() + " i SSF-s medlemsregister", "Meddelande", JOptionPane.INFORMATION_MESSAGE);
          return;
        }
      }
      catch (JSONException e) {
        JOptionPane.showMessageDialog(parent, "Kunde inte uppdatera spelare:" + e.getMessage(), "Fel", JOptionPane.WARNING_MESSAGE);
        return;
      }
    }
    // Find player given SSF ID
    try {
      String url = String.format("http://member.schack.se/JSONMemberInfo?memberid=%d", memberid);
      JSONObject obj = SSF.readJsonFromUrl(url);       
      player.setFirstName(obj.getString("firstName"));
      player.setLastName(obj.getString("lastName"));
      player.setSSFID(memberid);
      String mainClub = obj.getString("mainclub");
      Club club = settings.getClubs().getClub(mainClub);
      if (club == null) {
        settings.getClubs().addToDB(mainClub);
        club = settings.getClubs().getClub(mainClub);
      }
      player.setClub(club);
      player.setBirthdate(Date.valueOf(obj.getString("birthdate")));
      player.setFederation("SWE");
      player.setFIDENr(obj.getInt("fideid"));
      player.setTitle(obj.getString("fidetitle"));
      String sex = obj.getString("sex");
      player.setSex(sex.equalsIgnoreCase("F") ? "K" : sex);
      JSONArray ratings = obj.getJSONArray("lasks");
      if (ratings != null && ratings.length() > 0) {
        Date newestDate = new Date(0);
        for (int j = 0; j < ratings.length(); j++) {
          JSONObject rating = ratings.getJSONObject(j);
          Date date = Date.valueOf(rating.getString("date"));
          if (date.after(newestDate)) {
            player.setRatingN(rating.getInt("rating"));
            newestDate = date;
          }
        }
      }
      ratings = obj.getJSONArray("elos");
      if (ratings != null && ratings.length() > 0) {
        Date newestDate = new Date(0);
        for (int j = 0; j < ratings.length(); j++) {
          JSONObject rating = ratings.getJSONObject(j);
          Date date = Date.valueOf(rating.getString("date"));
          if (date.after(newestDate)) {
            player.setRatingI(rating.getInt("rating"));
            newestDate = date;
          }
        }
      }
      if (update) {
        if (tournament != null) {
          player.update(tournament);
        }
        else {
          player.update();
        }
      }
    }
    catch (JSONException e) {
      JOptionPane.showMessageDialog(parent, String.format("Kan inte hitta spelare %s %s med SSF-id=%d.\nKontrollera SSF-id-numret.", player.getFirstName(), player.getLastName(), player.getSSFID()), "SpelarID fel", JOptionPane.INFORMATION_MESSAGE);
    }
  }

  private static String readAll(Reader rd) throws IOException {
    StringBuilder sb = new StringBuilder();
    int cp;
    while ((cp = rd.read()) != -1) {
      sb.append((char)cp);
    }
    return sb.toString();
  }

  private static JSONObject readJsonFromUrl(String url) throws IOException, JSONException {
    URL ur = new URL(translate(url));
    InputStream is = ur.openStream();
    try {
      BufferedReader rd = new BufferedReader(new InputStreamReader(is, Charset.forName("UTF-8")));
      String jsonText = readAll(rd);
//      System.out.println(jsonText);
      JSONObject json = new JSONObject(jsonText);
      return json;
    }
    finally {
      is.close();
    }
  }
  
  private static String translate(String string) {
    return string.replace("", "a").replace("", "a").replace("", "o").replace("", "A").replace("", "A").replace("", "O");
  }  
}
